#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:t:s:"
#define TICKS_PER_SEC       100     /* for ultra sparc */
#define DEFAULT_WAIT_SEC    10

/********************************************************
**
** Test Module: SimpleWait.c
**
** Purpose:     Given a timeout value, wait for a
**              WAIT_EVENT notification. Some other
**              process needs to create the time-tag event.
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_WaitObj    WaitObj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];


    /*
     * default wait ( solaris sparc is 100 ticks/sec )
     */
    WaitObj.ticks = (DEFAULT_WAIT_SEC * TICKS_PER_SEC);

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 's' :
                WaitObj.ticks = ((atoi(optarg)) * TICKS_PER_SEC);
                break;
            case 't' :
                WaitObj.ticks = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     *  wait for external time-tag event
     */
    printf(" Waiting for event.......");
    fflush (stdout);

    if ( (rc = TPRO_waitEvent(hnd, &WaitObj)) != TPRO_SUCCESS )
    {
        char emsg[MAX_ERR_MESG_LENGTH];

        switch (rc)
        {
            case TPRO_TIMEOUT_ERR:
                (void)sprintf_s(emsg, MAX_ERR_MESG_LENGTH, " TIMEOUT_ERROR <%d> ticks ", 
                        WaitObj.ticks );
                errMsg = emsg;
                break;
            case TPRO_HANDLE_ERR:
                errMsg = "TPRO_HANDLE_ERR";
                break;
            case TPRO_COMM_ERR:
                errMsg = "TPRO_COMM_ERR";
                break;
            default:
                errMsg = "Unknown error";
                break;
        }
        (void) printf("\n%s: <%s> %s rc: <%d>.\n", 
                    pgmname, fullDevName, errMsg, rc);

        ret = 1;

    }
    else
    {
        /*
         * success:  print out the time
         */
        (void)printf("<%s> %s: received Days <%d> Hours <%d> Min <%d> Sec <%f>\n",
                    fullDevName, pgmname, WaitObj.days, 
                    WaitObj.hours, WaitObj.minutes, WaitObj.seconds);
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-t TICKS] [-s SECONDS] [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-t   # of ticks (100/sec)\n");
    (void)printf("\t-s   # of seconds (defaults to %d)\n", DEFAULT_WAIT_SEC );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
